@@ -145,6 +145,7 @@
 #include "clutter-debug.h"
 #include "clutter-enum-types.h"
 #include "clutter-interval.h"
+#include "clutter-marshal.h"
 #include "clutter-private.h"
 #include "clutter-scriptable.h"
 #include "clutter-script-private.h"
@@ -551,7 +552,7 @@ clutter_animation_class_init (ClutterAnimationClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (ClutterAnimationClass, started),
                   NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  _clutter_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
   /**
@@ -572,7 +573,7 @@ clutter_animation_class_init (ClutterAnimationClass *klass)
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (ClutterAnimationClass, completed),
                   NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  _clutter_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 }
 
@@ -1791,7 +1792,6 @@ clutter_animation_setupv (ClutterAnimation    *animation,
   ClutterAnimationPrivate *priv = animation->priv;
   ClutterAnimatable *animatable = NULL;
   GObjectClass *klass = NULL;
-  gboolean is_fixed = FALSE;
   gint i;
 
   if (CLUTTER_IS_ANIMATABLE (priv->object))
@@ -1803,6 +1803,7 @@ clutter_animation_setupv (ClutterAnimation    *animation,
     {
       const gchar *property_name = properties[i];
       GParamSpec *pspec;
+      gboolean is_fixed = FALSE;
 
       if (g_str_has_prefix (property_name, "fixed::"))
         {
