@@ -114,6 +114,8 @@
 #include "cogl/cogl.h"
 #include "pango/cogl-pango.h"
 
+#include "cally.h" /* For accessibility support */
+
 /* main context */
 static ClutterMainContext *ClutterCntx       = NULL;
 
@@ -128,6 +130,7 @@ static gboolean clutter_show_fps             = FALSE;
 static gboolean clutter_fatal_warnings       = FALSE;
 static gboolean clutter_disable_mipmap_text  = FALSE;
 static gboolean clutter_use_fuzzy_picking    = FALSE;
+static gboolean clutter_enable_accessibility = TRUE;
 
 static guint clutter_default_fps             = 60;
 
@@ -203,6 +206,24 @@ clutter_get_show_fps (void)
   return clutter_show_fps;
 }
 
+/**
+ * clutter_get_accessibility_enabled:
+ *
+ * Returns whether Clutter has accessibility support enabled.  As
+ * least, a value of TRUE means that there are a proper AtkUtil
+ * implementation available
+ *
+ * Return value: %TRUE if Clutter has accessibility support enabled
+ *
+ * Since: 1.4
+ */
+gboolean
+clutter_get_accessibility_enabled (void)
+{
+  return cally_get_cally_initialized ();
+}
+
+
 void
 _clutter_stage_maybe_relayout (ClutterActor *stage)
 {
@@ -551,7 +572,7 @@ _clutter_do_pick (ClutterStage   *stage,
 		  ClutterPickMode mode)
 {
   ClutterMainContext *context;
-  guchar              pixel[3] = { 0xff, 0xff, 0xff };
+  guchar              pixel[4] = { 0xff, 0xff, 0xff, 0xff };
   CoglColor           stage_pick_id;
   guint32             id;
   GLboolean           dither_was_on;
@@ -632,11 +653,17 @@ _clutter_do_pick (ClutterStage   *stage,
   /* Make sure Cogl flushes any batched geometry to the GPU driver */
   cogl_flush ();
 
-  /* Read the color of the screen co-ords pixel */
+  /* Read the color of the screen co-ords pixel. RGBA_8888_PRE is used
+     even though we don't care about the alpha component because under
+     GLES this is the only format that is guaranteed to work so Cogl
+     will end up having to do a conversion if any other format is
+     used. The format is requested as pre-multiplied because Cogl
+     assumes that all pixels in the framebuffer are premultiplied so
+     it avoids a conversion. */
   CLUTTER_TIMER_START (_clutter_uprof_context, pick_read);
   cogl_read_pixels (x, y, 1, 1,
                     COGL_READ_PIXELS_COLOR_BUFFER,
-                    COGL_PIXEL_FORMAT_RGB_888,
+                    COGL_PIXEL_FORMAT_RGBA_8888_PRE,
                     pixel);
   CLUTTER_TIMER_STOP (_clutter_uprof_context, pick_read);
 
@@ -672,6 +699,29 @@ result:
   return actor;
 }
 
+CoglPangoFontMap *
+_clutter_context_get_pango_fontmap (ClutterMainContext *self)
+{
+  CoglPangoFontMap *font_map;
+  gdouble resolution;
+  gboolean use_mipmapping;
+
+  if (G_LIKELY (self->font_map != NULL))
+    return self->font_map;
+
+  font_map = COGL_PANGO_FONT_MAP (cogl_pango_font_map_new ());
+
+  resolution = clutter_backend_get_resolution (self->backend);
+  cogl_pango_font_map_set_resolution (font_map, resolution);
+
+  use_mipmapping = !clutter_disable_mipmap_text;
+  cogl_pango_font_map_set_use_mipmapping (font_map, use_mipmapping);
+
+  self->font_map = font_map;
+
+  return self->font_map;
+}
+
 static ClutterTextDirection
 clutter_get_text_direction (void)
 {
@@ -768,22 +818,12 @@ _clutter_context_get_pango_context (ClutterMainContext *self)
 PangoContext *
 _clutter_context_create_pango_context (ClutterMainContext *self)
 {
+  CoglPangoFontMap *font_map;
   PangoContext *context;
 
-  if (self->font_map == NULL)
-    {
-      gdouble resolution;
+  font_map = _clutter_context_get_pango_fontmap (self);
 
-      self->font_map = COGL_PANGO_FONT_MAP (cogl_pango_font_map_new ());
-
-      resolution = clutter_backend_get_resolution (self->backend);
-      cogl_pango_font_map_set_resolution (self->font_map, resolution);
-
-      if (G_LIKELY (!clutter_disable_mipmap_text))
-        cogl_pango_font_map_set_use_mipmapping (self->font_map, TRUE);
-    }
-
-  context = cogl_pango_font_map_create_context (self->font_map);
+  context = cogl_pango_font_map_create_context (font_map);
   update_pango_context (self->backend, context);
   pango_context_set_language (context, pango_language_get_default ());
 
@@ -1572,6 +1612,10 @@ clutter_init_real (GError **error)
   clutter_is_initialized = TRUE;
   ctx->is_initialized = TRUE;
 
+  /* Initialize a11y */
+  if (clutter_enable_accessibility)
+    cally_accessibility_init ();
+
   return CLUTTER_INIT_SUCCESS;
 }
 
@@ -1603,6 +1647,8 @@ static GOptionEntry clutter_args[] = {
   { "clutter-no-profile", 0, 0, G_OPTION_ARG_CALLBACK, clutter_arg_no_profile_cb,
     N_("Clutter profiling flags to unset"), "FLAGS" },
 #endif /* CLUTTER_ENABLE_PROFILE */
+  { "clutter-enable-accessibility", 0, 0, G_OPTION_ARG_NONE, &clutter_enable_accessibility,
+    N_("Enable accessibility"), NULL },
   { NULL, },
 };
 
@@ -2854,8 +2900,10 @@ clutter_get_keyboard_grab (void)
 void
 clutter_clear_glyph_cache (void)
 {
-  if (CLUTTER_CONTEXT ()->font_map)
-    cogl_pango_font_map_clear_glyph_cache (CLUTTER_CONTEXT ()->font_map);
+  CoglPangoFontMap *font_map;
+
+  font_map = _clutter_context_get_pango_fontmap (CLUTTER_CONTEXT ());
+  cogl_pango_font_map_clear_glyph_cache (font_map);
 }
 
 /**
@@ -2876,17 +2924,19 @@ clutter_clear_glyph_cache (void)
 void
 clutter_set_font_flags (ClutterFontFlags flags)
 {
+  ClutterMainContext *context = _clutter_context_get_default ();
+  CoglPangoFontMap *font_map;
   ClutterFontFlags old_flags, changed_flags;
   const cairo_font_options_t *font_options;
   cairo_font_options_t *new_font_options;
+  gboolean use_mipmapping;
   ClutterBackend *backend;
 
   backend = clutter_get_default_backend ();
 
-  if (CLUTTER_CONTEXT ()->font_map)
-    cogl_pango_font_map_set_use_mipmapping (CLUTTER_CONTEXT ()->font_map,
-					    (flags
-                                             & CLUTTER_FONT_MIPMAPPING) != 0);
+  font_map = _clutter_context_get_pango_fontmap (context);
+  use_mipmapping = (flags & CLUTTER_FONT_MIPMAPPING) != 0;
+  cogl_pango_font_map_set_use_mipmapping (font_map, use_mipmapping);
 
   old_flags = clutter_get_font_flags ();
 
@@ -2907,8 +2957,9 @@ clutter_set_font_flags (ClutterFontFlags flags)
 
   cairo_font_options_destroy (new_font_options);
 
-  if (CLUTTER_CONTEXT ()->pango_context)
-    update_pango_context (backend, CLUTTER_CONTEXT ()->pango_context);
+  /* update the default pango context, if any */
+  if (context->pango_context != NULL)
+    update_pango_context (backend, context->pango_context);
 }
 
 /**
@@ -2924,18 +2975,16 @@ clutter_set_font_flags (ClutterFontFlags flags)
 ClutterFontFlags
 clutter_get_font_flags (void)
 {
-  ClutterMainContext *ctxt = CLUTTER_CONTEXT ();
+  ClutterMainContext *context = CLUTTER_CONTEXT ();
   CoglPangoFontMap *font_map = NULL;
   const cairo_font_options_t *font_options;
   ClutterFontFlags flags = 0;
 
-  font_map = CLUTTER_CONTEXT ()->font_map;
-
-  if (G_LIKELY (font_map)
-      && cogl_pango_font_map_get_use_mipmapping (font_map))
+  font_map = _clutter_context_get_pango_fontmap (context);
+  if (cogl_pango_font_map_get_use_mipmapping (font_map))
     flags |= CLUTTER_FONT_MIPMAPPING;
 
-  font_options = clutter_backend_get_font_options (ctxt->backend);
+  font_options = clutter_backend_get_font_options (context->backend);
 
   if ((cairo_font_options_get_hint_style (font_options)
        != CAIRO_HINT_STYLE_DEFAULT)
@@ -2991,10 +3040,9 @@ clutter_get_input_device_for_id (gint id)
 PangoFontMap *
 clutter_get_font_map (void)
 {
-  if (CLUTTER_CONTEXT ()->font_map)
-    return PANGO_FONT_MAP (CLUTTER_CONTEXT ()->font_map);
+  ClutterMainContext *context = _clutter_context_get_default ();
 
-  return NULL;
+  return PANGO_FONT_MAP (_clutter_context_get_pango_fontmap (context));
 }
 
 typedef struct _ClutterRepaintFunction
