@@ -217,13 +217,9 @@ enum
   PROP_0,
 
   PROP_FILENAME_SET,
-  PROP_FILENAME,
-
-  PROP_LAST
+  PROP_FILENAME
 };
 
-static GParamSpec *obj_props[PROP_LAST];
-
 #define CLUTTER_SCRIPT_GET_PRIVATE(obj) \
         (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CLUTTER_TYPE_SCRIPT, ClutterScriptPrivate))
 
@@ -401,11 +397,10 @@ clutter_script_class_init (ClutterScriptClass *klass)
    * Since: 0.6
    */
   pspec = g_param_spec_boolean ("filename-set",
-                                P_("Filename Set"),
-                                P_("Whether the :filename property is set"),
+                                "Filename Set",
+                                "Whether the :filename property is set",
                                 FALSE,
                                 CLUTTER_PARAM_READABLE);
-  obj_props[PROP_FILENAME_SET] = pspec;
   g_object_class_install_property (gobject_class, PROP_FILENAME_SET, pspec);
 
   /**
@@ -417,11 +412,10 @@ clutter_script_class_init (ClutterScriptClass *klass)
    * Since: 0.6
    */
   pspec = g_param_spec_string ("filename",
-                               P_("Filename"),
-                               P_("The path of the currently parsed file"),
+                               "Filename",
+                               "The path of the currently parsed file",
                                NULL,
                                CLUTTER_PARAM_READABLE);
-  obj_props[PROP_FILENAME] = pspec;
   g_object_class_install_property (gobject_class, PROP_FILENAME, pspec);
 }
 
