@@ -18,9 +18,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  *
  * ClutterTimeoutPool: pool of timeout functions using the same slice of
  *                     the GLib main loop
@@ -377,6 +377,10 @@ clutter_timeout_pool_new (gint priority)
   if (!source)
     return NULL;
 
+#if GLIB_CHECK_VERSION (2, 25, 8)
+  g_source_set_name (source, "Clutter timeout pool");
+#endif
+
   if (priority != G_PRIORITY_DEFAULT)
     g_source_set_priority (source, priority);
 
