@@ -26,12 +26,14 @@
 
 #include "cogl-internal.h"
 #include "cogl-context-driver.h"
+#include "cogl-context-winsys.h"
 #include "cogl-primitives.h"
 #include "cogl-clip-stack.h"
 #include "cogl-matrix-stack.h"
 #include "cogl-material-private.h"
 #include "cogl-atlas.h"
 #include "cogl-buffer-private.h"
+#include "cogl-bitmask.h"
 
 typedef struct
 {
@@ -43,14 +45,17 @@ typedef struct
 typedef struct
 {
   /* Features cache */
-  CoglFeatureFlags  feature_flags;
-  gboolean          features_cached;
+  CoglFeatureFlags        feature_flags;
+  CoglFeatureFlagsPrivate feature_flags_private;
+  gboolean                features_cached;
 
   CoglHandle        default_material;
+  CoglHandle        default_layer_0;
+  CoglHandle        default_layer_n;
+  CoglHandle        dummy_layer_dependant;
 
   /* Enable cache */
   unsigned long     enable_flags;
-  guint8            color_alpha;
 
   gboolean          enable_backface_culling;
   CoglFrontWinding  flushed_front_winding;
@@ -63,11 +68,18 @@ typedef struct
 
   /* Client-side matrix stack or NULL if none */
   CoglMatrixMode    flushed_matrix_mode;
-  GList            *texture_units;
+
+  GArray           *texture_units;
+  int               active_texture_unit;
+
+  CoglMaterialFogState legacy_fog_state;
 
   /* Materials */
-  CoglHandle        simple_material;
-  CoglHandle	    source_material;
+  CoglMaterial     *simple_material;
+  CoglMaterial     *source_material;
+  GString          *arbfp_source_buffer;
+
+  int               legacy_state_set;
 
   /* Textures */
   CoglHandle        default_gl_texture_2d_tex;
@@ -82,15 +94,34 @@ typedef struct
   GArray           *polygon_vertices;
 
   /* Some simple caching, to minimize state changes... */
-  CoglHandle	    current_material;
-  unsigned long     current_material_flags;
-  CoglMaterialFlushOptions current_material_flush_options;
-  GArray           *current_layers;
-  unsigned int      n_texcoord_arrays_enabled;
+  CoglMaterial     *current_material;
+  unsigned long     current_material_changes_since_flush;
+  gboolean          current_material_skip_gl_color;
 
-  /* PBOs */
-  /* This can be used to check if a pbo is bound */
-  CoglBuffer       *current_pbo;
+  GArray           *material0_nodes;
+  GArray           *material1_nodes;
+
+  /* Bitmask of texture coordinates arrays that are enabled */
+  CoglBitmask       texcoord_arrays_enabled;
+  /* These are temporary bitmasks that are used when disabling
+     texcoord arrays. They are here just to avoid allocating new ones
+     each time */
+  CoglBitmask       texcoord_arrays_to_disable;
+  CoglBitmask       temp_bitmask;
+
+  gboolean          gl_blend_enable_cache;
+
+  gboolean              depth_test_enabled_cache;
+  CoglDepthTestFunction depth_test_function_cache;
+  gboolean              depth_writing_enabled_cache;
+  float                 depth_range_near_cache;
+  float                 depth_range_far_cache;
+
+  gboolean              legacy_depth_test_enabled;
+
+  float             point_size_cache;
+
+  CoglBuffer       *current_buffer[COGL_BUFFER_BIND_TARGET_COUNT];
 
   /* Framebuffers */
   GSList           *framebuffer_stack;
@@ -99,13 +130,8 @@ typedef struct
   gboolean          dirty_gl_viewport;
 
   /* Primitives */
-  floatVec2         path_start;
-  floatVec2         path_pen;
-  GArray           *path_nodes;
-  unsigned int      last_path;
-  floatVec2         path_nodes_min;
-  floatVec2         path_nodes_max;
-  CoglHandle        stencil_material;
+  CoglHandle        current_path;
+  CoglMaterial     *stencil_material;
 
   /* Pre-generated VBOs containing indices to generate GL_TRIANGLES
      out of a vertex array of quads */
@@ -115,7 +141,7 @@ typedef struct
 
   gboolean          in_begin_gl_block;
 
-  CoglHandle        texture_download_material;
+  CoglMaterial     *texture_download_material;
 
   CoglAtlas        *atlas;
   CoglHandle        atlas_texture;
@@ -126,11 +152,28 @@ typedef struct
      chances of getting the same colour during an animation */
   guint8            journal_rectangles_color;
 
-  /* Cached value for GL_MAX_TEXTURE_UNITS to avoid calling
+  /* Cached values for GL_MAX_TEXTURE_[IMAGE_]UNITS to avoid calling
      glGetInteger too often */
   GLint             max_texture_units;
+  GLint             max_texture_image_units;
+  GLint             max_activateable_texture_units;
+
+  /* Fragment processing programs */
+  CoglHandle              current_program;
+
+  CoglMaterialProgramType current_use_program_type;
+  GLuint                  current_gl_program;
+
+  /* List of types that will be considered a subclass of CoglTexture in
+     cogl_is_texture */
+  GSList           *texture_types;
+
+  /* List of types that will be considered a subclass of CoglBuffer in
+     cogl_is_buffer */
+  GSList           *buffer_types;
 
   CoglContextDriver drv;
+  CoglContextWinsys winsys;
 } CoglContext;
 
 CoglContext *
