@@ -29,6 +29,7 @@
 #include "cogl-internal.h"
 #include "cogl-context.h"
 #include "cogl-handle.h"
+#include "cogl-object-private.h"
 #include "cogl-util.h"
 #include "cogl-texture-private.h"
 #include "cogl-framebuffer-private.h"
@@ -51,6 +52,8 @@
 #define glFramebufferRenderbuffer         ctx->drv.pf_glFramebufferRenderbuffer
 #define glCheckFramebufferStatus          ctx->drv.pf_glCheckFramebufferStatus
 #define glDeleteFramebuffers              ctx->drv.pf_glDeleteFramebuffers
+#define glGetFramebufferAttachmentParameteriv \
+                                          ctx->drv.pf_glGetFramebufferAttachmentParameteriv
 
 #endif
 
@@ -81,6 +84,24 @@
 #ifndef GL_DEPTH_COMPONENT16
 #define GL_DEPTH_COMPONENT16    0x81A5
 #endif
+#ifndef GL_FRAMEBUFFER_ATTACHMENT_RED_SIZE
+#define GL_FRAMEBUFFER_ATTACHMENT_RED_SIZE      0x8212
+#endif
+#ifndef GL_FRAMEBUFFER_ATTACHMENT_GREEN_SIZE
+#define GL_FRAMEBUFFER_ATTACHMENT_GREEN_SIZE    0x8213
+#endif
+#ifndef GL_FRAMEBUFFER_ATTACHMENT_BLUE_SIZE
+#define GL_FRAMEBUFFER_ATTACHMENT_BLUE_SIZE     0x8214
+#endif
+#ifndef GL_FRAMEBUFFER_ATTACHMENT_ALPHA_SIZE
+#define GL_FRAMEBUFFER_ATTACHMENT_ALPHA_SIZE    0x8215
+#endif
+#ifndef GL_FRAMEBUFFER_ATTCHMENT_DEPTH_SIZE
+#define GL_FRAMEBUFFER_ATTACHMENT_DEPTH_SIZE    0x8216
+#endif
+#ifndef GL_FRAMEBUFFER_ATTACHMENT_STENCIL_SIZE
+#define GL_FRAMEBUFFER_ATTACHMENT_STENCIL_SIZE  0x8217
+#endif
 
 typedef enum {
   _TRY_DEPTH_STENCIL = 1L<<0,
@@ -92,21 +113,22 @@ static void _cogl_framebuffer_free (CoglFramebuffer *framebuffer);
 static void _cogl_onscreen_free (CoglOnscreen *onscreen);
 static void _cogl_offscreen_free (CoglOffscreen *offscreen);
 
-COGL_HANDLE_DEFINE (Onscreen, onscreen);
-COGL_HANDLE_DEFINE (Offscreen, offscreen);
+COGL_OBJECT_INTERNAL_DEFINE (Onscreen, onscreen);
+COGL_OBJECT_DEFINE (Offscreen, offscreen);
+COGL_OBJECT_DEFINE_DEPRECATED_REF_COUNTING (offscreen);
 
 /* XXX:
- * The CoglHandle macros don't support any form of inheritance, so for
- * now we implement the CoglHandle support for the CoglFramebuffer
+ * The CoglObject macros don't support any form of inheritance, so for
+ * now we implement the CoglObject support for the CoglFramebuffer
  * abstract class manually.
  */
 
 gboolean
-cogl_is_framebuffer (CoglHandle handle)
+_cogl_is_framebuffer (void *object)
 {
-  CoglHandleObject *obj = (CoglHandleObject *)handle;
+  CoglHandleObject *obj = object;
 
-  if (handle == COGL_INVALID_HANDLE)
+  if (obj == NULL)
     return FALSE;
 
   return obj->klass->type == _cogl_handle_onscreen_get_type ()
@@ -130,14 +152,16 @@ _cogl_framebuffer_init (CoglFramebuffer *framebuffer,
   framebuffer->modelview_stack  = _cogl_matrix_stack_new ();
   framebuffer->projection_stack = _cogl_matrix_stack_new ();
 
+  framebuffer->dirty_bitmasks   = TRUE;
+
   /* Initialise the clip stack */
-  _cogl_clip_stack_state_init (&framebuffer->clip_state);
+  _cogl_clip_state_init (&framebuffer->clip_state);
 }
 
 void
 _cogl_framebuffer_free (CoglFramebuffer *framebuffer)
 {
-  _cogl_clip_stack_state_destroy (&framebuffer->clip_state);
+  _cogl_clip_state_destroy (&framebuffer->clip_state);
 
   _cogl_matrix_stack_destroy (framebuffer->modelview_stack);
   framebuffer->modelview_stack = NULL;
@@ -147,36 +171,30 @@ _cogl_framebuffer_free (CoglFramebuffer *framebuffer)
 }
 
 int
-_cogl_framebuffer_get_width (CoglHandle handle)
+_cogl_framebuffer_get_width (CoglFramebuffer *framebuffer)
 {
-  CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (handle);
   return framebuffer->width;
 }
 
 int
-_cogl_framebuffer_get_height (CoglHandle handle)
+_cogl_framebuffer_get_height (CoglFramebuffer *framebuffer)
 {
-  CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (handle);
   return framebuffer->height;
 }
 
-CoglClipStackState *
-_cogl_framebuffer_get_clip_state (CoglHandle handle)
+CoglClipState *
+_cogl_framebuffer_get_clip_state (CoglFramebuffer *framebuffer)
 {
-  CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (handle);
-
   return &framebuffer->clip_state;
 }
 
 void
-_cogl_framebuffer_set_viewport (CoglHandle handle,
+_cogl_framebuffer_set_viewport (CoglFramebuffer *framebuffer,
                                 int x,
                                 int y,
                                 int width,
                                 int height)
 {
-  CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (handle);
-
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
   if (framebuffer->viewport_x == x &&
@@ -197,37 +215,32 @@ _cogl_framebuffer_set_viewport (CoglHandle handle,
 }
 
 int
-_cogl_framebuffer_get_viewport_x (CoglHandle handle)
+_cogl_framebuffer_get_viewport_x (CoglFramebuffer *framebuffer)
 {
-  CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (handle);
   return framebuffer->viewport_x;
 }
 
 int
-_cogl_framebuffer_get_viewport_y (CoglHandle handle)
+_cogl_framebuffer_get_viewport_y (CoglFramebuffer *framebuffer)
 {
-  CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (handle);
   return framebuffer->viewport_y;
 }
 
 int
-_cogl_framebuffer_get_viewport_width (CoglHandle handle)
+_cogl_framebuffer_get_viewport_width (CoglFramebuffer *framebuffer)
 {
-  CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (handle);
   return framebuffer->viewport_width;
 }
 
 int
-_cogl_framebuffer_get_viewport_height (CoglHandle handle)
+_cogl_framebuffer_get_viewport_height (CoglFramebuffer *framebuffer)
 {
-  CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (handle);
   return framebuffer->viewport_height;
 }
 
 void
-_cogl_framebuffer_get_viewport4fv (CoglHandle handle, int *viewport)
+_cogl_framebuffer_get_viewport4fv (CoglFramebuffer *framebuffer, int *viewport)
 {
-  CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (handle);
   viewport[0] = framebuffer->viewport_x;
   viewport[1] = framebuffer->viewport_y;
   viewport[2] = framebuffer->viewport_width;
@@ -235,19 +248,81 @@ _cogl_framebuffer_get_viewport4fv (CoglHandle handle, int *viewport)
 }
 
 CoglMatrixStack *
-_cogl_framebuffer_get_modelview_stack (CoglHandle handle)
+_cogl_framebuffer_get_modelview_stack (CoglFramebuffer *framebuffer)
 {
-  CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (handle);
   return framebuffer->modelview_stack;
 }
 
 CoglMatrixStack *
-_cogl_framebuffer_get_projection_stack (CoglHandle handle)
+_cogl_framebuffer_get_projection_stack (CoglFramebuffer *framebuffer)
 {
-  CoglFramebuffer *framebuffer = COGL_FRAMEBUFFER (handle);
   return framebuffer->projection_stack;
 }
 
+static inline void
+_cogl_framebuffer_init_bits (CoglFramebuffer *framebuffer)
+{
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  if (G_LIKELY (!framebuffer->dirty_bitmasks))
+    return;
+
+#ifdef HAVE_COGL_GL
+  if (cogl_features_available (COGL_FEATURE_OFFSCREEN)
+      && framebuffer->type == COGL_FRAMEBUFFER_TYPE_OFFSCREEN)
+    {
+      GLenum attachment, pname;
+
+      attachment = GL_COLOR_ATTACHMENT0;
+
+      pname = GL_FRAMEBUFFER_ATTACHMENT_RED_SIZE;
+      GE( glGetFramebufferAttachmentParameteriv (GL_FRAMEBUFFER,
+                                                 attachment,
+                                                 pname,
+                                                 &framebuffer->red_bits) );
+
+      pname = GL_FRAMEBUFFER_ATTACHMENT_GREEN_SIZE;
+      GE( glGetFramebufferAttachmentParameteriv (GL_FRAMEBUFFER,
+                                                 attachment,
+                                                 pname,
+                                                 &framebuffer->green_bits) );
+
+      pname = GL_FRAMEBUFFER_ATTACHMENT_BLUE_SIZE;
+      GE( glGetFramebufferAttachmentParameteriv (GL_FRAMEBUFFER,
+                                                 attachment,
+                                                 pname,
+                                                 &framebuffer->blue_bits) );
+
+      pname = GL_FRAMEBUFFER_ATTACHMENT_ALPHA_SIZE;
+      GE( glGetFramebufferAttachmentParameteriv (GL_FRAMEBUFFER,
+                                                 attachment,
+                                                 pname,
+                                                 &framebuffer->alpha_bits) );
+    }
+  else
+#endif /* HAVE_COGL_GL */
+    {
+      GE( glGetIntegerv (GL_RED_BITS,   &framebuffer->red_bits)   );
+      GE( glGetIntegerv (GL_GREEN_BITS, &framebuffer->green_bits) );
+      GE( glGetIntegerv (GL_BLUE_BITS,  &framebuffer->blue_bits)  );
+      GE( glGetIntegerv (GL_ALPHA_BITS, &framebuffer->alpha_bits) );
+    }
+
+
+  COGL_NOTE (OFFSCREEN,
+             "RGBA Bits for framebuffer[%p, %s]: %d, %d, %d, %d",
+             framebuffer,
+             framebuffer->type == COGL_FRAMEBUFFER_TYPE_OFFSCREEN
+               ? "offscreen"
+               : "onscreen",
+             framebuffer->red_bits,
+             framebuffer->blue_bits,
+             framebuffer->green_bits,
+             framebuffer->alpha_bits);
+
+  framebuffer->dirty_bitmasks = FALSE;
+}
+
 static gboolean
 try_creating_fbo (CoglOffscreen *offscreen,
                   TryFBOFlags flags,
@@ -415,7 +490,7 @@ cogl_offscreen_new_to_texture (CoglHandle texhandle)
                               cogl_texture_get_width (texhandle),
                               cogl_texture_get_height (texhandle));
 
-      return _cogl_offscreen_handle_new (offscreen);
+      return _cogl_offscreen_object_new (offscreen);
     }
   else
     {
@@ -466,7 +541,7 @@ _cogl_onscreen_new (void)
                           0xdeadbeef, /* width */
                           0xdeadbeef); /* height */
 
-  return _cogl_onscreen_handle_new (onscreen);
+  return _cogl_onscreen_object_new (onscreen);
 }
 
 static void
@@ -530,21 +605,21 @@ _cogl_free_framebuffer_stack (GSList *stack)
 static void
 _cogl_set_framebuffer_real (CoglFramebuffer *framebuffer)
 {
-  CoglHandle *entry;
+  CoglFramebuffer **entry;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
   cogl_flush ();
 
-  entry = &ctx->framebuffer_stack->data;
+  entry = (CoglFramebuffer **)&ctx->framebuffer_stack->data;
 
   ctx->dirty_bound_framebuffer = 1;
   ctx->dirty_gl_viewport = 1;
 
   if (framebuffer != COGL_INVALID_HANDLE)
-    cogl_handle_ref (framebuffer);
+    cogl_object_ref (framebuffer);
   if (*entry != COGL_INVALID_HANDLE)
-    cogl_handle_unref (*entry);
+    cogl_object_unref (*entry);
 
   *entry = framebuffer;
 
@@ -554,16 +629,16 @@ _cogl_set_framebuffer_real (CoglFramebuffer *framebuffer)
    * we flush */
   _cogl_matrix_stack_dirty (framebuffer->modelview_stack);
   _cogl_matrix_stack_dirty (framebuffer->projection_stack);
-  _cogl_clip_stack_state_dirty (&framebuffer->clip_state);
+  _cogl_clip_state_dirty (&framebuffer->clip_state);
 }
 
 void
-cogl_set_framebuffer (CoglHandle handle)
+cogl_set_framebuffer (CoglFramebuffer *framebuffer)
 {
-  g_return_if_fail (cogl_is_framebuffer (handle));
+  g_return_if_fail (_cogl_is_framebuffer (framebuffer));
 
-  if (_cogl_get_framebuffer () != handle)
-    _cogl_set_framebuffer_real (COGL_FRAMEBUFFER (handle));
+  if (_cogl_get_framebuffer () != framebuffer)
+    _cogl_set_framebuffer_real (framebuffer);
 }
 
 /* XXX: deprecated API */
@@ -578,22 +653,22 @@ cogl_set_draw_buffer (CoglBufferTarget target, CoglHandle handle)
   cogl_set_framebuffer (handle);
 }
 
-CoglHandle
+CoglFramebuffer *
 _cogl_get_framebuffer (void)
 {
   _COGL_GET_CONTEXT (ctx, NULL);
 
   g_assert (ctx->framebuffer_stack);
 
-  return (CoglHandle)ctx->framebuffer_stack->data;
+  return ctx->framebuffer_stack->data;
 }
 
 void
-cogl_push_framebuffer (CoglHandle buffer)
+cogl_push_framebuffer (CoglFramebuffer *buffer)
 {
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  g_return_if_fail (cogl_is_framebuffer (buffer));
+  g_return_if_fail (_cogl_is_framebuffer (buffer));
   g_assert (ctx->framebuffer_stack);
 
   cogl_flush ();
@@ -614,8 +689,8 @@ cogl_push_draw_buffer (void)
 void
 cogl_pop_framebuffer (void)
 {
-  CoglHandle to_pop;
-  CoglHandle to_restore;
+  CoglFramebuffer *to_pop;
+  CoglFramebuffer *to_restore;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -627,7 +702,7 @@ cogl_pop_framebuffer (void)
 
   cogl_flush ();
 
-  cogl_handle_unref (to_pop);
+  cogl_object_unref (to_pop);
   ctx->framebuffer_stack =
     g_slist_remove_link (ctx->framebuffer_stack,
                          ctx->framebuffer_stack);
@@ -647,15 +722,11 @@ cogl_pop_draw_buffer (void)
 }
 
 void
-_cogl_framebuffer_flush_state (CoglHandle handle,
+_cogl_framebuffer_flush_state (CoglFramebuffer *framebuffer,
                                CoglFramebufferFlushFlags flags)
 {
-  CoglFramebuffer *framebuffer;
-
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  framebuffer = COGL_FRAMEBUFFER (handle);
-
   if (cogl_features_available (COGL_FEATURE_OFFSCREEN) &&
       ctx->dirty_bound_framebuffer)
     {
@@ -697,10 +768,15 @@ _cogl_framebuffer_flush_state (CoglHandle handle,
       ctx->dirty_gl_viewport = FALSE;
     }
 
+  /* since we might have changed the framebuffer, we should initialize
+   * the bits; this is a no-op if they have already been initialized
+   */
+  _cogl_framebuffer_init_bits (framebuffer);
+
   /* XXX: Flushing clip state may trash the modelview and projection
    * matrices so we must do it before flushing the matrices...
    */
-  _cogl_flush_clip_state (&framebuffer->clip_state);
+  _cogl_clip_state_flush (&framebuffer->clip_state);
 
   if (!(flags & COGL_FRAMEBUFFER_FLUSH_SKIP_MODELVIEW))
     _cogl_matrix_stack_flush_to_gl (framebuffer->modelview_stack,
@@ -710,3 +786,35 @@ _cogl_framebuffer_flush_state (CoglHandle handle,
                                   COGL_MATRIX_PROJECTION);
 }
 
+int
+_cogl_framebuffer_get_red_bits (CoglFramebuffer *framebuffer)
+{
+  _cogl_framebuffer_init_bits (framebuffer);
+
+  return framebuffer->red_bits;
+}
+
+int
+_cogl_framebuffer_get_green_bits (CoglFramebuffer *framebuffer)
+{
+  _cogl_framebuffer_init_bits (framebuffer);
+
+  return framebuffer->green_bits;
+}
+
+int
+_cogl_framebuffer_get_blue_bits (CoglFramebuffer *framebuffer)
+{
+  _cogl_framebuffer_init_bits (framebuffer);
+
+  return framebuffer->blue_bits;
+}
+
+int
+_cogl_framebuffer_get_alpha_bits (CoglFramebuffer *framebuffer)
+{
+  _cogl_framebuffer_init_bits (framebuffer);
+
+  return framebuffer->alpha_bits;
+}
+
