@@ -28,6 +28,8 @@
 #ifndef __COGL_MATERIAL_PRIVATE_H
 #define __COGL_MATERIAL_PRIVATE_H
 
+#include "cogl.h"
+
 #include "cogl-material.h"
 #include "cogl-matrix.h"
 #include "cogl-matrix-stack.h"
@@ -35,10 +37,6 @@
 
 #include <glib.h>
 
-typedef struct _CoglMaterial	      CoglMaterial;
-typedef struct _CoglMaterialLayer     CoglMaterialLayer;
-
-
 /*
  * cogl-material.c owns the GPU's texture unit state so we have some
  * private structures for describing the current state of a texture
@@ -143,16 +141,38 @@ _cogl_bind_gl_texture_transient (GLenum gl_target,
                                  gboolean is_foreign);
 
 #if defined (HAVE_COGL_GL)
-/* glsl, arbfp, fixed */
+
+/* NB: material->backend is currently a 3bit unsigned int bitfield */
+#define COGL_MATERIAL_BACKEND_GLSL       0
+#define COGL_MATERIAL_BACKEND_GLSL_MASK  (1L<<0)
+#define COGL_MATERIAL_BACKEND_ARBFP      1
+#define COGL_MATERIAL_BACKEND_ARBFP_MASK (1L<<1)
+#define COGL_MATERIAL_BACKEND_FIXED      2
+#define COGL_MATERIAL_BACKEND_FIXED_MASK (1L<<2)
+
 #define COGL_MATERIAL_N_BACKENDS         3
+
 #elif defined (HAVE_COGL_GLES2)
-/* glsl, fixed */
+
+#define COGL_MATERIAL_BACKEND_GLSL       0
+#define COGL_MATERIAL_BACKEND_GLSL_MASK  (1L<<0)
+#define COGL_MATERIAL_BACKEND_FIXED      1
+#define COGL_MATERIAL_BACKEND_FIXED_MASK (1L<<1)
+
 #define COGL_MATERIAL_N_BACKENDS         2
+
 #else /* HAVE_COGL_GLES */
-/* fixed */
+
+#define COGL_MATERIAL_BACKEND_FIXED      0
+#define COGL_MATERIAL_BACKEND_FIXED_MASK (1L<<0)
+
 #define COGL_MATERIAL_N_BACKENDS         1
+
 #endif
 
+#define COGL_MATERIAL_BACKEND_DEFAULT    0
+#define COGL_MATERIAL_BACKEND_UNDEFINED  3
+
 typedef enum
 {
   COGL_MATERIAL_LAYER_STATE_UNIT             = 1L<<0,
@@ -164,7 +184,9 @@ typedef enum
   COGL_MATERIAL_LAYER_STATE_COMBINE_CONSTANT = 1L<<5,
   COGL_MATERIAL_LAYER_STATE_USER_MATRIX      = 1L<<6,
 
-  /* COGL_MATERIAL_LAYER_STATE_TEXTURE_INTERN   = 1L<<7, */
+  COGL_MATERIAL_LAYER_STATE_POINT_SPRITE_COORDS = 1L<<7,
+
+  /* COGL_MATERIAL_LAYER_STATE_TEXTURE_INTERN   = 1L<<8, */
 
   COGL_MATERIAL_LAYER_STATE_ALL_SPARSE =
     COGL_MATERIAL_LAYER_STATE_UNIT |
@@ -173,12 +195,14 @@ typedef enum
     COGL_MATERIAL_LAYER_STATE_WRAP_MODES |
     COGL_MATERIAL_LAYER_STATE_COMBINE |
     COGL_MATERIAL_LAYER_STATE_COMBINE_CONSTANT |
-    COGL_MATERIAL_LAYER_STATE_USER_MATRIX,
+    COGL_MATERIAL_LAYER_STATE_USER_MATRIX |
+    COGL_MATERIAL_LAYER_STATE_POINT_SPRITE_COORDS,
 
   COGL_MATERIAL_LAYER_STATE_NEEDS_BIG_STATE =
     COGL_MATERIAL_LAYER_STATE_COMBINE |
     COGL_MATERIAL_LAYER_STATE_COMBINE_CONSTANT |
-    COGL_MATERIAL_LAYER_STATE_USER_MATRIX
+    COGL_MATERIAL_LAYER_STATE_USER_MATRIX |
+    COGL_MATERIAL_LAYER_STATE_POINT_SPRITE_COORDS,
 
 } CoglMaterialLayerState;
 
@@ -199,8 +223,52 @@ typedef struct
   /* The texture matrix dscribes how to transform texture coordinates */
   CoglMatrix matrix;
 
+  gboolean point_sprite_coords;
+
 } CoglMaterialLayerBigState;
 
+/* Materials and layers represent their state in a tree structure where
+ * some of the state relating to a given material or layer may actually
+ * be owned by one if is ancestors in the tree. We have a common data
+ * type to track the tree heirachy so we can share code... */
+typedef struct _CoglMaterialNode CoglMaterialNode;
+struct _CoglMaterialNode
+{
+  /* the parent in terms of class hierarchy, so anything inheriting
+   * from CoglMaterialNode also inherits from CoglObject. */
+  CoglObject _parent;
+
+  /* The parent material/layer */
+  CoglMaterialNode *parent;
+
+  /* As an optimization for creating leaf node materials/layers (the
+   * most common) we don't require any list node allocations to link
+   * to a single descendant. */
+  CoglMaterialNode *first_child;
+
+  /* Determines if node->first_child and node->children are
+   * initialized pointers. */
+  gboolean has_children;
+
+  /* Materials and layers are sparse structures defined as a diff
+   * against their parent and may have multiple children which depend
+   * on them to define the values of properties which they don't
+   * change. */
+  GList *children;
+};
+
+#define COGL_MATERIAL_NODE(X) ((CoglMaterialNode *)(X))
+
+typedef void (*CoglMaterialNodeUnparentVFunc) (CoglMaterialNode *node);
+
+typedef gboolean (*CoglMaterialNodeChildCallback) (CoglMaterialNode *child,
+                                                   void *user_data);
+
+void
+_cogl_material_node_foreach_child (CoglMaterialNode *node,
+                                   CoglMaterialNodeChildCallback callback,
+                                   void *user_data);
+
 struct _CoglMaterialLayer
 {
   /* XXX: Please think twice about adding members that *have* be
@@ -217,8 +285,11 @@ struct _CoglMaterialLayer
    * layers or if instead it can go under ->big_state.
    */
 
-  /* the parent in terms of class hierarchy */
-  CoglHandleObject   _parent;
+  /* Layers represent their state in a tree structure where some of
+   * the state relating to a given material or layer may actually be
+   * owned by one if is ancestors in the tree. We have a common data
+   * type to track the tree heirachy so we can share code... */
+  CoglMaterialNode _parent;
 
   /* Some layers have a material owner, which is to say that the layer
    * is referenced in that materials->layer_differences list.  A layer
@@ -226,21 +297,6 @@ struct _CoglMaterialLayer
    * other layers that keeps track of some shared state. */
   CoglMaterial      *owner;
 
-  /* Layers are sparse structures defined as a diff against
-   * their parent... */
-  CoglMaterialLayer *parent;
-
-  /* As an optimization for creating leaf node layers (the most
-   * common) we don't require any list node allocations to link
-   * to a single descendant. */
-  CoglMaterialLayer *first_child;
-
-  /* Layers are sparse structures defined as a diff against
-   * their parent and may have multiple children which depend
-   * on them to define the values of properties which they don't
-   * change. */
-  GList             *children;
-
   /* The lowest index is blended first then others on top */
   int	             index;
 
@@ -290,10 +346,6 @@ struct _CoglMaterialLayer
 
   /* bitfields */
 
-  /* Determines if layer->first_child and layer->children are
-   * initialized pointers. */
-  unsigned int          has_children:1;
-
   /* Determines if layer->big_state is valid */
   unsigned int          has_big_state:1;
 
@@ -312,8 +364,10 @@ typedef enum _CoglMaterialState
   COGL_MATERIAL_STATE_BLEND             = 1L<<5,
   COGL_MATERIAL_STATE_USER_SHADER       = 1L<<6,
   COGL_MATERIAL_STATE_DEPTH             = 1L<<7,
+  COGL_MATERIAL_STATE_FOG               = 1L<<8,
+  COGL_MATERIAL_STATE_POINT_SIZE        = 1L<<9,
 
-  COGL_MATERIAL_STATE_REAL_BLEND_ENABLE = 1L<<8,
+  COGL_MATERIAL_STATE_REAL_BLEND_ENABLE = 1L<<10,
 
   COGL_MATERIAL_STATE_ALL_SPARSE =
     COGL_MATERIAL_STATE_COLOR |
@@ -323,7 +377,9 @@ typedef enum _CoglMaterialState
     COGL_MATERIAL_STATE_ALPHA_FUNC |
     COGL_MATERIAL_STATE_BLEND |
     COGL_MATERIAL_STATE_USER_SHADER |
-    COGL_MATERIAL_STATE_DEPTH,
+    COGL_MATERIAL_STATE_DEPTH |
+    COGL_MATERIAL_STATE_FOG |
+    COGL_MATERIAL_STATE_POINT_SIZE,
 
   COGL_MATERIAL_STATE_AFFECTS_BLENDING =
     COGL_MATERIAL_STATE_COLOR |
@@ -338,7 +394,9 @@ typedef enum _CoglMaterialState
     COGL_MATERIAL_STATE_ALPHA_FUNC |
     COGL_MATERIAL_STATE_BLEND |
     COGL_MATERIAL_STATE_USER_SHADER |
-    COGL_MATERIAL_STATE_DEPTH
+    COGL_MATERIAL_STATE_DEPTH |
+    COGL_MATERIAL_STATE_FOG |
+    COGL_MATERIAL_STATE_POINT_SIZE
 
 } CoglMaterialState;
 
@@ -400,6 +458,16 @@ typedef struct
   float                 depth_range_far;
 } CoglMaterialDepthState;
 
+typedef struct
+{
+  gboolean        enabled;
+  CoglColor       color;
+  CoglFogMode     mode;
+  float           density;
+  float           z_near;
+  float           z_far;
+} CoglMaterialFogState;
+
 typedef struct
 {
   CoglMaterialLightingState lighting_state;
@@ -407,6 +475,8 @@ typedef struct
   CoglMaterialBlendState blend_state;
   CoglHandle user_program;
   CoglMaterialDepthState depth_state;
+  CoglMaterialFogState fog_state;
+  float point_size;
 } CoglMaterialBigState;
 
 typedef enum
@@ -437,33 +507,17 @@ struct _CoglMaterial
    * materials or if instead it can go under ->big_state.
    */
 
-  /* the parent in terms of class hierarchy */
-  CoglHandleObject _parent;
+  /* Layers represent their state in a tree structure where some of
+   * the state relating to a given material or layer may actually be
+   * owned by one if is ancestors in the tree. We have a common data
+   * type to track the tree heirachy so we can share code... */
+  CoglMaterialNode _parent;
 
   /* We need to track if a material is referenced in the journal
    * because we can't allow modification to these materials without
    * flushing the journal first */
   unsigned long    journal_ref_count;
 
-  /* Materials are sparse structures defined as a diff against
-   * their parent. */
-  CoglMaterial    *parent;
-
-  /* As an optimization for creating leaf node materials (the most
-   * common) we don't require any list node allocations to link
-   * to a single descendant.
-   *
-   * Only valid if ->has_children bitfield is set */
-  CoglMaterial    *first_child;
-
-  /* Materials are sparse structures defined as a diff against
-   * their parent and may have multiple children which depend
-   * on them to define the values of properties which they don't
-   * change.
-   *
-   * Only valid if ->has_children bitfield is set */
-  GList           *children;
-
   /* A mask of which sparse state groups are different in this
    * material in comparison to its parent. */
   unsigned long    differences;
@@ -554,10 +608,6 @@ struct _CoglMaterial
    * blending, this holds our final decision */
   unsigned int          real_blend_enable:1;
 
-  /* Determines if material->first_child and material->children are
-   * initialized pointers. */
-  unsigned int          has_children:1;
-
   unsigned int          layers_cache_dirty:1;
   unsigned int          deprecated_get_layers_list_dirty:1;
 
@@ -623,10 +673,10 @@ _cogl_material_init_default_layers (void);
  */
 
 gboolean
-_cogl_material_get_real_blend_enabled (CoglHandle handle);
+_cogl_material_get_real_blend_enabled (CoglMaterial *material);
 
 gboolean
-_cogl_material_layer_has_user_matrix (CoglHandle layer_handle);
+_cogl_material_layer_has_user_matrix (CoglMaterialLayer *layer);
 
 /*
  * Calls the pre_paint method on the layer texture if there is
@@ -634,7 +684,7 @@ _cogl_material_layer_has_user_matrix (CoglHandle layer_handle);
  * filter settings.
  */
 void
-_cogl_material_layer_pre_paint (CoglHandle layer_handler);
+_cogl_material_layer_pre_paint (CoglMaterialLayer *layerr);
 
 /*
  * CoglMaterialFlushFlag:
@@ -724,47 +774,65 @@ typedef struct _CoglMaterialFlushOptions
   CoglMaterialWrapModeOverrides wrap_mode_overrides;
 } CoglMaterialFlushOptions;
 
-void
-_cogl_material_get_colorubv (CoglHandle  handle,
-                             guint8     *color);
 
 void
-_cogl_material_flush_gl_state (CoglHandle material,
+_cogl_set_active_texture_unit (int unit_index);
+
+void
+_cogl_delete_gl_texture (GLuint gl_texture);
+
+int
+_cogl_get_max_texture_image_units (void);
+
+
+void
+_cogl_use_program (CoglHandle program_handle, CoglMaterialProgramType type);
+
+unsigned int
+_cogl_get_n_args_for_combine_func (GLint func);
+
+
+CoglMaterial *
+_cogl_material_get_parent (CoglMaterial *material);
+
+void
+_cogl_material_get_colorubv (CoglMaterial *material,
+                             guint8       *color);
+
+void
+_cogl_material_flush_gl_state (CoglMaterial *material,
                                gboolean skip_gl_state);
 
 gboolean
-_cogl_material_equal (CoglHandle material0_handle,
-                      CoglHandle material1_handle,
+_cogl_material_equal (CoglMaterial *material0,
+                      CoglMaterial *material1,
                       gboolean skip_gl_color);
 
-CoglHandle
-_cogl_material_journal_ref (CoglHandle material_handle);
+CoglMaterial *
+_cogl_material_journal_ref (CoglMaterial *material);
 
 void
-_cogl_material_journal_unref (CoglHandle material_handle);
+_cogl_material_journal_unref (CoglMaterial *material);
 
 /* TODO: These should be made public once we add support for 3D
    textures in Cogl */
 void
-_cogl_material_set_layer_wrap_mode_r (CoglHandle material,
+_cogl_material_set_layer_wrap_mode_r (CoglMaterial *material,
                                       int layer_index,
                                       CoglMaterialWrapMode mode);
 
 CoglMaterialWrapMode
-_cogl_material_layer_get_wrap_mode_r (CoglHandle layer);
+_cogl_material_layer_get_wrap_mode_r (CoglMaterialLayer *layer);
 
 void
-_cogl_material_set_user_program (CoglHandle handle,
+_cogl_material_set_user_program (CoglMaterial *material,
                                  CoglHandle program);
 
-void
-_cogl_delete_gl_texture (GLuint gl_texture);
-
 void
 _cogl_material_texture_storage_change_notify (CoglHandle texture);
 
 void
-_cogl_material_apply_legacy_state (CoglHandle handle);
+_cogl_material_apply_legacy_state (CoglMaterial *material);
 
 void
 _cogl_gl_use_program_wrapper (GLuint program);
@@ -774,18 +842,44 @@ _cogl_material_apply_overrides (CoglMaterial *material,
                                 CoglMaterialFlushOptions *options);
 
 CoglMaterialBlendEnable
-_cogl_material_get_blend_enabled (CoglHandle handle);
+_cogl_material_get_blend_enabled (CoglMaterial *material);
 
 void
-_cogl_material_set_blend_enabled (CoglHandle handle,
+_cogl_material_set_blend_enabled (CoglMaterial *material,
                                   CoglMaterialBlendEnable enable);
 
 void
-_cogl_material_set_static_breadcrumb (CoglHandle handle,
+_cogl_material_set_static_breadcrumb (CoglMaterial *material,
                                       const char *breadcrumb);
 
 unsigned long
-_cogl_material_get_age (CoglHandle handle);
+_cogl_material_get_age (CoglMaterial *material);
+
+CoglMaterial *
+_cogl_material_get_authority (CoglMaterial *material,
+                              unsigned long difference);
+
+unsigned long
+_cogl_material_layer_compare_differences (CoglMaterialLayer *layer0,
+                                          CoglMaterialLayer *layer1);
+
+CoglMaterialLayer *
+_cogl_material_layer_get_authority (CoglMaterialLayer *layer,
+                                    unsigned long difference);
+
+CoglHandle
+_cogl_material_layer_get_texture (CoglMaterialLayer *layer);
+
+typedef gboolean (*CoglMaterialLayerCallback) (CoglMaterialLayer *layer,
+                                               void *user_data);
+
+void
+_cogl_material_foreach_layer (CoglMaterial *material,
+                              CoglMaterialLayerCallback callback,
+                              void *user_data);
+
+int
+_cogl_material_layer_get_unit_index (CoglMaterialLayer *layer);
 
 #endif /* __COGL_MATERIAL_PRIVATE_H */
 
