@@ -159,7 +159,6 @@
 #define glDeleteBuffers ctx->drv.pf_glDeleteBuffers
 #define glMapBuffer ctx->drv.pf_glMapBuffer
 #define glUnmapBuffer ctx->drv.pf_glUnmapBuffer
-#define glActiveTexture ctx->drv.pf_glActiveTexture
 #define glClientActiveTexture ctx->drv.pf_glClientActiveTexture
 #ifndef GL_ARRAY_BUFFER
 #define GL_ARRAY_BUFFER GL_ARRAY_BUFFER_ARB
@@ -213,6 +212,7 @@ static void _cogl_vertex_buffer_free (CoglVertexBuffer *buffer);
 static void _cogl_vertex_buffer_indices_free (CoglVertexBufferIndices *buffer_indices);
 
 COGL_HANDLE_DEFINE (VertexBuffer, vertex_buffer);
+COGL_OBJECT_DEFINE_DEPRECATED_REF_COUNTING (vertex_buffer);
 COGL_HANDLE_DEFINE (VertexBufferIndices, vertex_buffer_indices);
 
 CoglHandle
@@ -800,7 +800,7 @@ filter_strided_attribute (CoglVertexBufferAttrib *attribute,
 	}
     }
   new_cogl_vbo = g_slice_alloc (sizeof (CoglVertexBufferVBO));
-  new_cogl_vbo->vbo_name = 0;
+  new_cogl_vbo->vbo_name = NULL;
   new_cogl_vbo->attributes = NULL;
   new_cogl_vbo->attributes =
     g_list_prepend (new_cogl_vbo->attributes, attribute);
@@ -1059,7 +1059,7 @@ upload_gl_vbo (CoglVertexBufferVBO *cogl_vbo)
 
   if (!fallback)
     {
-      g_return_if_fail (cogl_vbo->vbo_name != 0);
+      g_return_if_fail (cogl_vbo->vbo_name != NULL);
 
       GE (glBindBuffer (GL_ARRAY_BUFFER,
 			GPOINTER_TO_UINT (cogl_vbo->vbo_name)));
@@ -1352,7 +1352,7 @@ cogl_vertex_buffer_submit_real (CoglVertexBuffer *buffer)
    */
 
   new_multipack_vbo = g_slice_alloc (sizeof (CoglVertexBufferVBO));
-  new_multipack_vbo->vbo_name = 0;
+  new_multipack_vbo->vbo_name = NULL;
   new_multipack_vbo->flags =
     COGL_VERTEX_BUFFER_VBO_FLAG_MULTIPACK
     | COGL_VERTEX_BUFFER_VBO_FLAG_INFREQUENT_RESUBMIT;
@@ -1400,7 +1400,7 @@ cogl_vertex_buffer_submit_real (CoglVertexBuffer *buffer)
 	   * in their own VBO so that updates don't impact other attributes
 	   */
 
-	  cogl_vbo->vbo_name = 0;
+	  cogl_vbo->vbo_name = NULL;
 	  cogl_vbo->flags =
             COGL_VERTEX_BUFFER_VBO_FLAG_UNSTRIDED
 	    | COGL_VERTEX_BUFFER_VBO_FLAG_FREQUENT_RESUBMIT;
@@ -1509,7 +1509,7 @@ get_gl_type_from_attribute_flags (CoglVertexBufferAttribFlags flags)
     }
 }
 
-static void
+static CoglHandle
 enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
 {
   GList       *tmp;
@@ -1518,18 +1518,27 @@ enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
   GLuint       generic_index = 0;
 #endif
   unsigned long enable_flags = 0;
-  int          max_texcoord_attrib_unit = -1;
   const GList *layers;
   guint32      fallback_layers = 0;
-  guint32      disable_layers = ~0;
   int          i;
+  gboolean     skip_gl_color = FALSE;
   CoglMaterialFlushOptions options;
+  CoglHandle   source;
 
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+  _COGL_GET_CONTEXT (ctx, COGL_INVALID_HANDLE);
+
+  source = ctx->source_material;
 
   if (buffer->new_attributes)
     cogl_vertex_buffer_submit_real (buffer);
 
+  options.flags =
+    COGL_MATERIAL_FLUSH_FALLBACK_MASK;
+  memset (&options.wrap_mode_overrides, 0,
+          sizeof (options.wrap_mode_overrides));
+
+  _cogl_bitmask_clear_all (&ctx->temp_bitmask);
+
   for (tmp = buffer->submitted_vbos; tmp != NULL; tmp = tmp->next)
     {
       CoglVertexBufferVBO *cogl_vbo = tmp->data;
@@ -1541,7 +1550,7 @@ enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
 	{
 	  GE (glBindBuffer (GL_ARRAY_BUFFER,
 			    GPOINTER_TO_UINT (cogl_vbo->vbo_name)));
-	  base = 0;
+	  base = NULL;
 	}
       else
 	base = cogl_vbo->vbo_name;
@@ -1567,13 +1576,22 @@ enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
 	  switch (type)
 	    {
 	    case COGL_VERTEX_BUFFER_ATTRIB_FLAG_COLOR_ARRAY:
-	      enable_flags |= COGL_ENABLE_COLOR_ARRAY | COGL_ENABLE_BLEND;
+	      enable_flags |= COGL_ENABLE_COLOR_ARRAY;
 	      /* GE (glEnableClientState (GL_COLOR_ARRAY)); */
 	      pointer = (const GLvoid *)(base + attribute->u.vbo_offset);
 	      GE (glColorPointer (attribute->n_components,
 				  gl_type,
 				  attribute->stride,
 				  pointer));
+
+              if (!_cogl_material_get_real_blend_enabled (ctx->source_material))
+                {
+                  CoglMaterialBlendEnable blend_enable =
+                    COGL_MATERIAL_BLEND_ENABLE_ENABLED;
+                  source = cogl_material_copy (ctx->source_material);
+                  _cogl_material_set_blend_enabled (source, blend_enable);
+                  skip_gl_color = TRUE;
+                }
 	      break;
 	    case COGL_VERTEX_BUFFER_ATTRIB_FLAG_NORMAL_ARRAY:
 	      /* FIXME: go through cogl cache to enable normal array */
@@ -1592,9 +1610,8 @@ enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
 				     gl_type,
 				     attribute->stride,
 				     pointer));
-              if (attribute->texture_unit > max_texcoord_attrib_unit)
-                max_texcoord_attrib_unit = attribute->texture_unit;
-              disable_layers &= ~(1 << attribute->texture_unit);
+              _cogl_bitmask_set (&ctx->temp_bitmask,
+                                 attribute->texture_unit, TRUE);
 	      break;
 	    case COGL_VERTEX_BUFFER_ATTRIB_FLAG_VERTEX_ARRAY:
 	      enable_flags |= COGL_ENABLE_VERTEX_ARRAY;
@@ -1630,9 +1647,9 @@ enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
 	}
     }
 
-  layers = cogl_material_get_layers (ctx->source_material);
+  layers = cogl_material_get_layers (source);
   for (tmp = (GList *)layers, i = 0;
-       tmp != NULL && i <= max_texcoord_attrib_unit;
+       tmp != NULL;
        tmp = tmp->next, i++)
     {
       CoglHandle layer = (CoglHandle)tmp->data;
@@ -1643,11 +1660,49 @@ enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
       if (tex_handle == COGL_INVALID_HANDLE)
         continue;
 
+      if (!cogl_material_get_layer_point_sprite_coords_enabled (source, i))
+        {
+          /* By default COGL_MATERIAL_WRAP_MODE_AUTOMATIC becomes
+             GL_CLAMP_TO_EDGE but we want GL_REPEAT to maintain
+             compatibility with older versions of Cogl so we'll
+             override it. We don't want to do this for point sprites
+             because in that case the whole texture is drawn so you
+             would usually want clamp-to-edge. */
+          if (cogl_material_layer_get_wrap_mode_s (layer) ==
+              COGL_MATERIAL_WRAP_MODE_AUTOMATIC)
+            {
+              options.wrap_mode_overrides.values[i].s =
+                COGL_MATERIAL_WRAP_MODE_OVERRIDE_REPEAT;
+              options.flags |= COGL_MATERIAL_FLUSH_WRAP_MODE_OVERRIDES;
+            }
+          if (cogl_material_layer_get_wrap_mode_t (layer) ==
+              COGL_MATERIAL_WRAP_MODE_AUTOMATIC)
+            {
+              options.wrap_mode_overrides.values[i].t =
+                COGL_MATERIAL_WRAP_MODE_OVERRIDE_REPEAT;
+              options.flags |= COGL_MATERIAL_FLUSH_WRAP_MODE_OVERRIDES;
+            }
+          if (_cogl_material_layer_get_wrap_mode_r (layer) ==
+              COGL_MATERIAL_WRAP_MODE_AUTOMATIC)
+            {
+              options.wrap_mode_overrides.values[i].r =
+                COGL_MATERIAL_WRAP_MODE_OVERRIDE_REPEAT;
+              options.flags |= COGL_MATERIAL_FLUSH_WRAP_MODE_OVERRIDES;
+            }
+        }
+
       /* Give the texture a chance to know that we're rendering
          non-quad shaped primitives. If the texture is in an atlas it
          will be migrated */
       _cogl_texture_ensure_non_quad_rendering (tex_handle);
 
+      /* We need to ensure the mipmaps are ready before deciding
+       * anything else about the texture because the texture storate
+       * could completely change if it needs to be migrated out of the
+       * atlas and will affect how we validate the layer.
+       */
+      _cogl_material_layer_pre_paint (layer);
+
       if (!_cogl_texture_can_hardware_repeat (tex_handle))
         {
           g_warning ("Disabling layer %d of the current source material, "
@@ -1670,36 +1725,79 @@ enable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
         }
     }
 
-  for (i = max_texcoord_attrib_unit + 1; i < ctx->n_texcoord_arrays_enabled; i++)
-    {
-      GE (glClientActiveTexture (GL_TEXTURE0 + i));
-      GE (glDisableClientState (GL_TEXTURE_COORD_ARRAY));
-    }
-  ctx->n_texcoord_arrays_enabled = max_texcoord_attrib_unit + 1;
+  /* Disable any tex coord arrays that we didn't use */
+  _cogl_disable_other_texcoord_arrays (&ctx->temp_bitmask);
 
   /* NB: _cogl_framebuffer_flush_state may disrupt various state (such
    * as the material state) when flushing the clip stack, so should
    * always be done first when preparing to draw. */
   _cogl_framebuffer_flush_state (_cogl_get_framebuffer (), 0);
 
-  options.flags =
-    COGL_MATERIAL_FLUSH_FALLBACK_MASK |
-    COGL_MATERIAL_FLUSH_DISABLE_MASK;
   options.fallback_layers = fallback_layers;
-  options.disable_layers = disable_layers;
 
-  _cogl_material_flush_gl_state (ctx->source_material, &options);
-  enable_flags |= _cogl_material_get_cogl_enable_flags (ctx->source_material);
+  if (G_UNLIKELY (options.flags))
+    {
+      /* If we haven't already created a derived material... */
+      if (source == ctx->source_material)
+        source = cogl_material_copy (ctx->source_material);
+      _cogl_material_apply_overrides (source, &options);
+
+      /* TODO:
+       * overrides = cogl_material_get_data (material,
+       *                                     last_overrides_key);
+       * if (overrides)
+       *   {
+       *     age = cogl_material_get_age (material);
+       *     XXX: actually we also need to check for legacy_state
+       *     and blending overrides for use of glColorPointer...
+       *     if (overrides->ags != age ||
+       *         memcmp (&overrides->options, &options,
+       *                 sizeof (options) != 0)
+       *       {
+       *         cogl_handle_unref (overrides->weak_material);
+       *         g_slice_free (Overrides, overrides);
+       *         overrides = NULL;
+       *       }
+       *   }
+       * if (!overrides)
+       *   {
+       *     overrides = g_slice_new (Overrides);
+       *     overrides->weak_material =
+       *       cogl_material_weak_copy (ctx->source_material);
+       *     _cogl_material_apply_overrides (overrides->weak_material,
+       *                                     &options);
+       *
+       *     cogl_material_set_data (material, last_overrides_key,
+       *                             weak_overrides,
+       *                             free_overrides_cb,
+       *                             NULL);
+       *   }
+       * source = overrides->weak_material;
+       */
+    }
+
+  if (G_UNLIKELY (ctx->legacy_state_set))
+    {
+      /* If we haven't already created a derived material... */
+      if (source == ctx->source_material)
+        source = cogl_material_copy (ctx->source_material);
+      _cogl_material_apply_legacy_state (source);
+    }
+
+  _cogl_material_flush_gl_state (source, skip_gl_color);
 
   if (ctx->enable_backface_culling)
     enable_flags |= COGL_ENABLE_BACKFACE_CULLING;
 
-  cogl_enable (enable_flags);
+  _cogl_enable (enable_flags);
   _cogl_flush_face_winding ();
+
+  return source;
 }
 
 static void
-disable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
+disable_state_for_drawing_buffer (CoglVertexBuffer *buffer,
+                                  CoglHandle source)
 {
   GList *tmp;
   GLenum gl_type;
@@ -1709,6 +1807,9 @@ disable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
+  if (G_UNLIKELY (source != ctx->source_material))
+    cogl_handle_unref (source);
+
   /* Disable all the client state that cogl doesn't currently know
    * about:
    */
@@ -1740,9 +1841,11 @@ disable_state_for_drawing_buffer (CoglVertexBuffer *buffer)
 	      GE (glDisableClientState (GL_NORMAL_ARRAY));
 	      break;
 	    case COGL_VERTEX_BUFFER_ATTRIB_FLAG_TEXTURE_COORD_ARRAY:
-              GE (glClientActiveTexture (GL_TEXTURE0 +
-                                         attribute->texture_unit));
-	      GE (glDisableClientState (GL_TEXTURE_COORD_ARRAY));
+              /* The enabled state of the texture coord arrays is
+                 cached in ctx->enabled_texcoord_arrays so we don't
+                 need to do anything here. The array will be disabled
+                 by the next drawing primitive if it is not
+                 required */
 	      break;
 	    case COGL_VERTEX_BUFFER_ATTRIB_FLAG_VERTEX_ARRAY:
 	      /* GE (glDisableClientState (GL_VERTEX_ARRAY)); */
@@ -1767,6 +1870,7 @@ cogl_vertex_buffer_draw (CoglHandle       handle,
 		         int              count)
 {
   CoglVertexBuffer *buffer;
+  CoglHandle source;
 
   if (!cogl_is_vertex_buffer (handle))
     return;
@@ -1775,11 +1879,11 @@ cogl_vertex_buffer_draw (CoglHandle       handle,
 
   buffer = _cogl_vertex_buffer_pointer_from_handle (handle);
 
-  enable_state_for_drawing_buffer (buffer);
+  source = enable_state_for_drawing_buffer (buffer);
 
   GE (glDrawArrays (mode, first, count));
 
-  disable_state_for_drawing_buffer (buffer);
+  disable_state_for_drawing_buffer (buffer, source);
 }
 
 static int
@@ -1808,7 +1912,7 @@ cogl_vertex_buffer_indices_new (CoglIndicesType  indices_type,
   gsize indices_bytes;
   CoglVertexBufferIndices *indices;
 
-  _COGL_GET_CONTEXT (ctx, 0);
+  _COGL_GET_CONTEXT (ctx, NULL);
 
   indices = g_slice_alloc (sizeof (CoglVertexBufferIndices));
 
@@ -1828,7 +1932,7 @@ cogl_vertex_buffer_indices_new (CoglIndicesType  indices_type,
     {
       g_critical ("unknown indices type %d", indices_type);
       g_slice_free (CoglVertexBufferIndices, indices);
-      return 0;
+      return NULL;
     }
 
   indices_bytes = get_indices_type_size (indices->type) * indices_len;
@@ -1903,6 +2007,7 @@ cogl_vertex_buffer_draw_elements (CoglHandle       handle,
     (cogl_get_features () & COGL_FEATURE_VBOS) ? FALSE : TRUE;
   gsize byte_offset;
   CoglVertexBufferIndices *indices = NULL;
+  CoglHandle source;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -1918,7 +2023,7 @@ cogl_vertex_buffer_draw_elements (CoglHandle       handle,
 
   indices = _cogl_vertex_buffer_indices_pointer_from_handle (indices_handle);
 
-  enable_state_for_drawing_buffer (buffer);
+  source = enable_state_for_drawing_buffer (buffer);
 
   byte_offset = indices_offset * get_indices_type_size (indices->type);
   if (fallback)
@@ -1930,7 +2035,7 @@ cogl_vertex_buffer_draw_elements (CoglHandle       handle,
   GE (glDrawRangeElements (mode, min_index, max_index,
                            count, indices->type, (void *)byte_offset));
 
-  disable_state_for_drawing_buffer (buffer);
+  disable_state_for_drawing_buffer (buffer, source);
 
   GE (glBindBuffer (GL_ELEMENT_ARRAY_BUFFER, 0));
 }
