@@ -100,10 +100,16 @@ _cogl_features_init (void)
 
 #ifdef HAVE_COGL_GLES2
   flags |= COGL_FEATURE_SHADERS_GLSL | COGL_FEATURE_OFFSCREEN;
+  /* Note GLES 2 core doesn't support mipmaps for npot textures or
+   * repeat modes other than CLAMP_TO_EDGE. */
+  flags |= COGL_FEATURE_TEXTURE_NPOT_BASIC;
 #endif
 
   flags |= COGL_FEATURE_VBOS;
 
+  /* Both GLES 1.1 and GLES 2.0 support point sprites in core */
+  flags |= COGL_FEATURE_POINT_SPRITE;
+
   /* Cache features */
   ctx->feature_flags = flags;
   ctx->features_cached = TRUE;
